/* This file is part of dc_qt.

    dc_qt is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    dc_qt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dc_qt; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "dctc_connection.h"
#include "util.h"
#include "dc_settings.h"
#include "debugdlg.h"

#include <string>

#include <qsocketnotifier.h>

#include <unistd.h>
#include <signal.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/un.h>
#include <errno.h>
#include <stdio.h>
#include <qstringlist.h>
#include <qprocess.h>

using namespace std;

const int MAX_WAIT = 180;

dctc_connection::dctc_connection(QObject *parent, const char *name)
	: QObject(parent, name)
{
	pid = 0;
}

void dctc_connection::kill_client()
{
	pid = 0;
	dctcproc->kill();
	
/*
 * The preferred method of killing dctc is by sending
 * /FORCEQUIT
 * so all this crap is no longer necessary
 *
	if (pid == 0)
		return;

	QString debug;
	debug.sprintf("killing pid: %d", pid);
	log(debug);
	kill(pid, SIGQUIT);

	// if the client doesn't respond to a quit, repeatedly try to kill it
	while (waitpid(pid, NULL, WNOHANG) == -1)
	{
		usleep( (1000000/180)*20);
		kill(pid, SIGKILL);
	}
	
	pid = 0;
*/
}

#define GET_SETTING(x, y) \
if (settings.get_setting(x) != QString::null) \
{ \
	arglst += y; \
	arglst += settings.get_setting(x); \
}


// handles the forking of the client to start the dctc text client
// this crashes a lot, I have no idea why.  Maybe sleep_on_client isn't
// working properly
int dctc_connection::fork_client(const char *hubip)
{
     QStringList arglst;

     arglst += "dctc";
     arglst += "-l";
     arglst += "-k";
     arglst += "-t";
     arglst += "-g";
     arglst += hubip;
     GET_SETTING(dc_settings::NICKNAME, "-n");
     GET_SETTING(dc_settings::DESCRIPTION, "-i");
     GET_SETTING(dc_settings::EMAIL,"-e");
     GET_SETTING(dc_settings::SLOTS, "-d");
     GET_SETTING(dc_settings::OFFSET, "-o");
     GET_SETTING(dc_settings::ADDRESS, "-a");
     GET_SETTING(dc_settings::PORT, "-p");
     if( settings.get_setting(dc_settings::USEPROXY)=="true")
     {
	  if( settings.get_setting(dc_settings::USESOCKS)=="true")
	  {
	       arglst += "--socksaddr=" + settings.get_setting(dc_settings::PROXYADDRESS);
	       arglst += "--socksport=" + settings.get_setting(dc_settings::PROXYPORT);
	       GET_SETTING(dc_settings::PROXYUSERNAME,"-U");
	       GET_SETTING(dc_settings::PROXYPWD,"-K");
	  }
	  else
	  {
	       arglst += "--connectproxy=" + settings.get_setting(dc_settings::PROXYADDRESS) + ":" + settings.get_setting(dc_settings::PROXYPORT);
	  }
     }

     if(settings.get_setting(dc_settings::CONNECTION)!=QString::null) 
     {
	  arglst += "-c";
	  arglst += settings.get_setting(dc_settings::CONNECTION);
     }
     arglst += "--precmd";
     arglst += "/RECOND 30";
     arglst += "--precmd";
     arglst += "/REBUILD 900";
     arglst += "--precmd";
     arglst += "/LPATH " + settings.get_setting(dc_settings::DLDIR);
     // bra eller inte?
     arglst += "--precmd";
     arglst += "/DDL";
     arglst += "--precmd";
     arglst += "/FOLLOWFORCE";

     // TODO: proper error handling here!
     if( settings.get_setting(dc_settings::DLDIR) == QString::null )
	  debugWin->print("dctc_connection: ERROR: no DLDIR specified, this is gonna fuck.\n");
     
     for (int j = 0; j < settings.num_shares(); j++)
     {
	  arglst += "-s";
	  arglst += settings.get_share(j);
     }
     
     if (settings.get_setting(dc_settings::PASSIVE) == "true")
	  arglst += "-f";

     // Print the arglist
     for ( QStringList::Iterator it = arglst.begin(); it != arglst.end(); ++it )
	  debugWin->print(*it + " ");
     debugWin->print("\n");

     dctcproc = new QProcess(arglst,this,"dctcprocess");
     if(!dctcproc->start())
     {
	  debugWin->print("Error starting dctc\n");
	  return SLEEP_TIMED_OUT;
     }

     connect(dctcproc,SIGNAL(processExited()), this, SIGNAL(hub_died()));
     connect(dctcproc,SIGNAL(readyReadStderr()), this, SLOT(hub_stderr()));

     // we use the pid later to kill the correct dctc client
     pid = dctcproc->processIdentifier();

     sleep(1);          // annars funkar det inte fr det kommer send_command fort som fan
     
     return connect_to_running(hubip);
}

void dctc_connection::hub_stderr()
{
     if( dctcproc->canReadLineStderr() )
	  debugWin->print( "From dctc-stderr: " + dctcproc->readLineStderr() + "\n");
}

int dctc_connection::connect_to_running(const char *hubip, int new_pid)
{	
	QString debug;
	debug.sprintf("pid = %d", new_pid);

	if (new_pid)
	     pid = new_pid;

	// We use dctc_fd later when creating the QSocketDevice
	dctc_fd = socket(PF_UNIX, SOCK_STREAM, 0);

	if (dctc_fd == -1)
	{
	     return COULD_NOT_CREATE_SOCKET;
	}


	name.sun_family = AF_UNIX;

	char pidhex[9];
	sprintf(pidhex, "%08X", pid);
	
	// we open the socket created by dctc in $HOME/.dctc/running/dctc-xxxxx-yyyyy
	// where xxxxx == the pid of the client in hex
	// yyyyy == the ip of the hub
	QString socket_path = settings.get_home_dir() + "/.dctc/running/";
	socket_path += "dctc-";
	socket_path += pidhex;
	socket_path += "-";
	socket_path += hubip;
	strcpy(name.sun_path, socket_path.latin1());
	
	int retcode = ::connect(dctc_fd, (sockaddr*)&name, sizeof(struct sockaddr_un));
	dctc_socket = new QSocketDevice( dctc_fd, QSocketDevice::Stream );

	if (retcode == -1)
	{
	     debugWin->print("Could not connect to socket " + socket_path + "\n");
	     perror("connect");
	     kill_client();
	     return COULD_NOT_CONNECT_TO_SOCKET;
	}
	
	// the notifier calls dataReceived whenever the socket has something to read
        read_notifier = new QSocketNotifier(dctc_fd, QSocketNotifier::Read);
	QObject::connect(read_notifier, SIGNAL(activated(int)), this, SLOT(dataReceived(int)));

	return CONNECT_SUCCESS;
}

// handler for receiving data through the socket
void dctc_connection::dataReceived(int)
{
	// if there is no running dctc, then we can't receive anything
	if (!pid)
	{
		QObject::disconnect(this, SLOT(dataReceived(int)));
		return;
	}

	Q_LONG n;

	// Get the available amount of data
	n = dctc_socket->bytesAvailable();
	char c[n+1];
	
	if( dctc_socket->readBlock( c, n ) == -1 )
	{
	     debugWin->print("Error while reading from socket\n");
	     return;
	}
	c[n] = '\0';
	message = c;

#ifdef _DEBUG
//	cerr << "Message from dctc: " + message + "\n";
#endif
	message.replace(0xd,"\n");

	// deal with the message that we got back
	emit dctc_data(message);
}

// send the string command to the text client
// the required newline is added by the function
void dctc_connection::send_command(const QString &command)
{
	QString cmd(command + "\n");
	
	int ret = dctc_socket->writeBlock( cmd, cmd.length() );

	// everything ok
	if (ret == cmd.length())
	     return;
	
//	if(ret == -1) cerr << "Tried to write to invalid socket\n";
	//else cerr << "All the data was not written\n";

}

bool dctc_connection::sleep_on_client(const QString& socket)
{
	struct stat st;

	for (int i = 0; i < MAX_WAIT; i++)
	{
		if (stat(socket.latin1(), &st) == 0)
			return true;

		if (waitpid(pid, NULL, WNOHANG) == -1)
			return false;

		usleep( (1000000/180)*20);
	}

	return false;
}

int dctc_connection::hubconnect(const QString &hubip, int new_pid)
{
     // we can reconnect to this dctc later, so don't disconnect
//	if (pid)
//		hubdisconnect();

  if (new_pid)
    return connect_to_running(hubip.latin1(), new_pid);
	
  // handle the starting of the dctc text client
  int ret = fork_client(hubip.latin1());
  if (ret != CONNECT_SUCCESS)
    {
	 debugWin->print("Could not connect");
      kill_client();
    }
  
  return ret;
  
  // retrieve the userlist
  send_command("/ULIST");
}

// disconnect from this hub and kill the associated dctc client
void dctc_connection::hubdisconnect()
{
	send_command("/FORCEQUIT");
	delete read_notifier;
	delete dctc_socket;
	delete dctcproc;
	pid = 0;
}

void dctc_connection::data_to_send(const QString &cmd)
{
	send_command(cmd);
}
