/*
  Copyright 2003 Rikard Bjrklind, Mikael Gransell

  This file is part of dc-qt.

  dc-qt is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  dc-qt is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with dc-qt; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mainwdgt.h"
#include <qpushbutton.h>
#include <qmenubar.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qpopupmenu.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qtoolbutton.h>
#include <qsplitter.h>
#include <qstatusbar.h>
#include <qdialog.h>
#include <qlayout.h>
#include <qaction.h>

#include "dc_settings.h"
#include "transferdlg.h"
#include "preferences_dlg.h"
#include "searchwdgt.h"
#include "pixmaps.h"
#include "config.h"

DebugDlg* debugWin;
dc_settings settings;
Pixmap *pixmaps;
MainWdgt::MainWdgt( QWidget* parent, const char* name ) : QMainWindow( parent, name )
{
     pixmaps = new Pixmap;
//     QVBox *mainbox = new QVBox(this, "mainvbox");
     QSplitter *mainbox = new QSplitter(Qt::Vertical,this,"mainsplitter");

     hubTabs = new QTabWidget( mainbox, "Hub Tabs" );
     QObject::connect(hubTabs, SIGNAL(currentChanged(QWidget*)), this, SLOT(selectHub(QWidget*)));

     // Create the global transferWindow
     transferWin = new TransferDlg(mainbox,"Transfer window");

     QPopupMenu* file = new QPopupMenu( this, "File menu" );
     QPopupMenu* help = new QPopupMenu( this, "Help menu" );

     // Set our popupmenus in the main window menubar
     menuBar()->insertItem( "&Actions", file );
     menuBar()->insertItem( "&Help", help );

     QIconSet::setIconSize(QIconSet::Large,QSize(22,22));
     QIconSet newIcon(pixmaps->getNew());
     QIconSet closeIcon(pixmaps->getClose());
     QIconSet connectIcon(pixmaps->getConnect());
     QIconSet disIcon(pixmaps->getDisconnect());
     QIconSet searchIcon(pixmaps->getSearch());
     QIconSet configureIcon(pixmaps->getConfigure());
     QIconSet exitIcon(pixmaps->getExit());

     QAction *aNewHub = new QAction("New Hub", newIcon, "&New hub", CTRL+Key_N, this, "newhubtab");
     QAction *aCloseHub = new QAction("Close Hub", closeIcon, "C&lose hub", CTRL+Key_X, this, "closehubtab");
     aConnect = new QAction("Connect", connectIcon, "&Connect", CTRL+Key_C, this, "connect");
     aDisconnect = new QAction("Disconnect", disIcon, "&Disconnect", CTRL+Key_D, this, "disconnect");
     QAction *aSearch = new QAction("Search", searchIcon, "&Search", CTRL+Key_S, this, "search");
     QAction *aConfig = new QAction("Preferences", configureIcon, "&Preferences", CTRL+Key_P, this, "prefs");
     QAction *aExit = new QAction("Quit", exitIcon, "&Quit", CTRL+Key_Q,this, "quit");
     QObject::connect(aNewHub, SIGNAL(activated()), this, SLOT(newHubTab()));
     QObject::connect(aCloseHub, SIGNAL(activated()), this, SLOT(closeHubTab()));
     QObject::connect(aConnect, SIGNAL(activated()), this, SLOT(connectHub()));
     QObject::connect(aDisconnect, SIGNAL(activated()), this, SLOT(disHub()));
     QObject::connect(aSearch, SIGNAL(activated()), this, SLOT(searchHub()));
     QObject::connect(aConfig, SIGNAL(activated()), this, SLOT(showPreferences()));
     QObject::connect(aExit, SIGNAL(activated()), this, SLOT(close()));
     aNewHub->addTo(file);
     aCloseHub->addTo(file);
     aConnect->addTo(file);
     aDisconnect->addTo(file);
     aDisconnect->setEnabled(false);
     aSearch->addTo(file);
     aConfig->addTo(file);
     aExit->addTo(file);
     
     help->insertItem( "&About", this, SLOT(displayAbout()) );
     help->insertItem( "About &Qt",this, SLOT(displayAboutQt()) );

     newHubTab();
     
     // Add all things to our main window
     setCaption( "dc-qt " DCQT_VERSION );
     setCentralWidget( mainbox );

     // Create the toolbar
     QToolBar * tools = new QToolBar( this, "toolbar" );
     tools->setHorizontallyStretchable(true);
     tools->setLabel( "Tools" );

     setUsesBigPixmaps(true);
     setUsesTextLabel(true);

     aNewHub->addTo(tools);
     aCloseHub->addTo(tools);
     aSearch->addTo(tools);
     aConnect->addTo(tools);
     aDisconnect->addTo(tools);
//     aConfig->addTo(tools);
     aExit->addTo(tools);

//     tools->boxLayout()->insertStretch(-3);
     // Create the global debugWindow

     debugWin = new DebugDlg();
#ifndef QT_NO_DEBUG
     debugWin->show();
#endif

     switch(settings.init())
     {
     case dc_settings::SUCCESS:
	  break;
     case dc_settings::NO_SETTINGS:
	  showPreferences();
	  break;
     case dc_settings::ERROR_LOADING_PREFS:
	  QMessageBox::critical(this, "Error", "Error loading preferences file.", "Ok");
	  break;
//     case dc_settings::ERROR_LOADING_FAVOURITES:
//	  QMessageBox::critical(this, "Error", "Error loading the favourites file.","Ok");
//	  break;
     default:
	  // This should not happen
	  break;
     }
     QFont font;
     font.fromString( settings.get_setting(dc_settings::FONT) );
     QApplication::setFont(font,true); 

     statusBar()->message("Ready.",2000);
     statusBar()->setSizeGripEnabled(true);
}

void MainWdgt::setHubName(const QString &name, QWidget *wdgt)
{
     hubTabs->changeTab(wdgt,name);
}

void MainWdgt::showPreferences()
{
     preferences_dlg *pdlg = new preferences_dlg(this,"preferencesdialog",true);
     pdlg->exec();
     delete pdlg;
}

void MainWdgt::displayAbout()
{
     QMessageBox::about(this,"About dcqt","This is dc-qt " DCQT_VERSION "\ndc-qt is a front end for the direct connect client dctc.\nAuthors:\n\nRikard Bjrklind (olof@linux.nu)\nMikael Gransell \n\nhttp://dc-qt.sourceforge.net");
}

void MainWdgt::displayAboutQt()
{
     QMessageBox::aboutQt(this);
}

void MainWdgt::newHubTab()
{
     HubWdgt* hub = new HubWdgt( transferWin, hubTabs, "hub" );
     // Add our new hub
     hubTabs->addTab( hub, "New Hub" );
     // Set the id of our tab so we can remove it later
     hub->setId( hubTabs->indexOf(hub) );
     QObject::connect( hub, SIGNAL(gotName(const QString&, QWidget*)), this, SLOT(setHubName(const QString&, QWidget*)));
     QObject::connect( hub, SIGNAL(multiSearchResult(const QString&,const QString&,const QString&,const QString&,const QString&)),
		       this, SLOT(dispatchSearchResult(const QString&,const QString&,const QString&,const QString&,const QString&)));
     QObject::connect ( hub, SIGNAL(disconnected(int)), this, SLOT(hubDisconnected(int)));
     hub->show();
}

void MainWdgt::hubDisconnected(int id)
{
    if(hubTabs->currentPageIndex()==id)
    {
	aConnect->setEnabled(true);
	aDisconnect->setEnabled(false);
    }
    statusBar()->message("Hub " + hubTabs->label(id) + " disconnected");
}

void MainWdgt::dispatchSearchResult(const QString& u,const QString& f,const QString& s,const QString& sl,const QString& hubname)
{
    // Find the right hub
    for(int i=0;i < hubTabs->count();i++)
	if( hubTabs->label(i) == hubname ) // found it
	{
	    HubWdgt *w = (HubWdgt *)hubTabs->page(i);
	    w->handleSearchResult(u,f,s,sl,hubname);
	}
}

void MainWdgt::closeHubTab(  )
{
     HubWdgt* hub = (HubWdgt*)hubTabs->currentPage();
     if(!hub) return;
     if(hub->isConnected())
	  switch( QMessageBox::warning( this, "dc-qt",
					"Closing the hub tab will disconnect you from the hub and abort all transfers.\n",
					"Ok",
					"Cancel", 0, 0, 1 ) ) 
	  {
	  case 0: // The user clicked the Ok again button or pressed Enter
	       hubTabs->removePage( hub );
	       hub->disconnect();
	       delete hub;  
	       break;
	  case 1: // The user clicked the Cancel button or pressed Escape
	       break;
	       
	  }
     else 
     {
	  hubTabs->removePage( hub );
	  hub->disconnect();
	  delete hub;  
     }
}

/// Connects the current hub
void MainWdgt::connectHub()
{
     HubWdgt* hub = (HubWdgt*)hubTabs->currentPage();
     if(!hub) return;
     if(hub->connect())
     {
	  aConnect->setEnabled(false);
	  aDisconnect->setEnabled(true);
     }
}

/// Disconnects the current hub
void MainWdgt::disHub()
{
     HubWdgt* hub = (HubWdgt*)hubTabs->currentPage();
     if(!hub) return;
     hub->disconnect();
     aConnect->setEnabled(true);
     aDisconnect->setEnabled(false);
}

/// Brings up the search dialog for the active hub
void MainWdgt::searchHub()
{
     HubWdgt* hub = (HubWdgt*)hubTabs->currentPage();
     if(!hub) return;
     hub->openSearchDlg();
}

void MainWdgt::closeEvent( QCloseEvent *e)
{
     bool ask = false;
     // Check if there are any connected hubs
     for(int i=0; i < hubTabs->count();i++)
     {
	  HubWdgt *hub = (HubWdgt*)hubTabs->page(i);
	  if( hub->isConnected() ) ask = true;
     }

     if( ask )
     {
	  switch( QMessageBox::warning( this, "dc-qt",
					"Do you want to disconnect from all hubs before exiting?.\n",
					"Quit && disconnect",
					"Just Quit",
					"Cancel", 0, 2 ) ) 
	  {
	  case 0: 
	       for(int i=0; i < hubTabs->count();i++)
	       {
		    HubWdgt *hub = (HubWdgt*)hubTabs->page(i);
		    hub->disconnect();
	       }
	       e->accept();
	       break;
	  case 1:
	       e->accept();
	       break;
	  case 2:
	       e->ignore();
	  }
     }
     else e->accept();
}

void MainWdgt::selectHub(QWidget *w)
{
     HubWdgt *h = (HubWdgt *)w;
     if(h->isConnected())
     {
	  aConnect->setEnabled(false);
	  aDisconnect->setEnabled(true);
     }
     else {
	  aConnect->setEnabled(true);
	  aDisconnect->setEnabled(false);
     }

}

/*
 *
 * $Log:
 *
 */
