/*
  Copyright 2003 Rikard Bjrklind, Mikael Gransell

  This file is part of dc-qt.

  dc-qt is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  dc-qt is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with dc-qt; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "transferdlg.h"
#include "file_transfer_lv.h"
#include "queue_item.h"
#include "debugdlg.h"
#include <qlayout.h>
#include <qregexp.h>

void TransferDlg::menuClicked(int i)
{
    if (!lastClicked)
	return;
    
    if (i == 0) // retry
	emit retryTransfer( lastClicked->File().section('\\',-1,-1) );
    if (i == 1) // cancel
    {
	emit cancelTransfer( lastClicked->GDL() );
	delete lastClicked;
	lastClicked=0;
    }
}

void TransferDlg::showQueueMenu(QListViewItem* i, const QPoint &p, int)
{
    if(i) 
    {
	queueMenu->popup(p);
	lastClicked = (QueueItem*)i;
    }
}

TransferDlg::TransferDlg( QWidget* parent, 
			  const char* name ) : QWidget( parent, name )
{
    QGridLayout* layout = new QGridLayout( this, 1, 1 );
    tabs = new QTabWidget( this, "Transfer tabs" );
    downloads = new file_transfer_lv(DOWNLOAD,this,"Download list");
    uploads = new file_transfer_lv(UPLOAD,this,"Upload list");
    
    queue = new QListView(this,"Queue list");
    queue->addColumn("User");
    queue->addColumn("File");
    queue->addColumn("Size");
    queue->addColumn("%");
    
    connect (queue, SIGNAL(rightButtonClicked(QListViewItem*, const QPoint&, int)),
	     SLOT(showQueueMenu(QListViewItem*, const QPoint&, int)));

     queueMenu = new QPopupMenu(this, "QueuePopupMenu");
     queueMenu->insertItem("Activate", 0);
     queueMenu->insertItem("Cancel", 1);
     connect (queueMenu, SIGNAL(activated(int)), SLOT(menuClicked(int)));
     lastClicked = 0;

     history = new QListView(this,"Transfer History lv");
     history->addColumn("User");
     history->addColumn("File");

     tabs->addTab( downloads, "Downloads");
     tabs->addTab( uploads, "Uploads");
     tabs->addTab( queue, "Queue");
     tabs->addTab( history, "History");
     layout->addWidget(tabs,0,0);

     //     connect( this, SIGNAL(addTransfer(int, const QString&)), downloads, SLOT( add_transfer(int, const QString&)));
     connect( this,SIGNAL(addTransfer(int,const QString&, const QString&)), 
	      uploads, SLOT( add_transfer(int,const QString&, const QString&)));

     connect( this,
	      SIGNAL(transferInfo(int,const QString&,int,int,int,const QString&,const QString&)), 
	      downloads, 
	      SLOT(transfer_info(int,const QString&,int,int,int,const QString&,const QString&)));

     connect( this,SIGNAL(transferInfo(int,int,int)), uploads, SLOT(transfer_info(int,int,int)));
//     connect( this,SIGNAL(activeTransfers(list<int>&)), downloads, SLOT(active_transfers(list<int>&)));
     connect( downloads, SIGNAL(update_dlist()), this, SIGNAL(updateDList()));

     connect( downloads, SIGNAL(cancel_transfer( int )), this, SIGNAL(cancelTransfer( int )));
     connect( uploads, SIGNAL(cancel_transfer( int )), this, SIGNAL(cancelTransfer( int )));

     //     connect( this, SIGNAL(removeTransfer( int )), downloads, SLOT(removeTransfer( int )));
     connect( this, SIGNAL(removeTransfer( const QString&, const QString& )), uploads, SLOT(removeTransfer( const QString&,const QString& )));

     connect( downloads, SIGNAL(transferStart(const QString&, const QString&)), this, SLOT(dequeue(const QString&, const QString&)));
     connect( downloads, SIGNAL(transferDone(const QString&, const QString&)), 
	      this, SLOT( updateHistory(const QString&, const QString&)) );

     connect(this, SIGNAL(removeInactive()), downloads, SLOT(removeInactive()));

     connect(downloads,SIGNAL(get_flist(const QString&)),this,SIGNAL(getUserFileList(const QString&)));

}

void TransferDlg::updateHistory(const QString &user, const QString &file)
{
     new QListViewItem(history, user, file);
}

void TransferDlg::enqueue(int gdl, const QString& user, const QString& file, const QString& size)
{
     QListViewItem *i = queue->firstChild();

     // Strip the path
     QString f = file;
     int index = f.findRev('\\');
     f.remove(0,index+1);

     while(i!=0)
     {
	  if( i->text(0) == user && i->text(1) == f )
	       break;
	  i = i->itemBelow();
     } 

     if(!i)
	  new QueueItem(queue,gdl,user,f,size,0);
}

void TransferDlg::dequeue(const QString& user, const QString& file)
{
     debugWin->print("user="+user+" file="+file+"\n");
     QListViewItem *i = queue->firstChild();
     while(i!=0)
     {
	  if( i->text(0) == user && i->text(1) == file )
	       break;
	  i = i->itemBelow();
     } 

     if(i) queue->takeItem(i);
}



void TransferDlg::handleInfo(int gdl,const QString &fname,int size,int recSize,int last10,const QString& users,const QString& status)
{
    // Check if there are any W in status, insert them into the queue.
    // Theese are removed by the download filetransferlv when the signal transferInfo is received.

     QStringList llusers = QStringList::split('|',users);
     QStringList llstatus = QStringList::split('|',status);
     
     debugWin->print("transferdlg::handleInfo()\n");

     for(unsigned int i=0;i < llusers.size();i++)
     {
	 debugWin->print("transferdlg: examining user " + llusers[i] + "\n");
	  if( llstatus[i] == "W" )
	  {
	       // See if this item is already in the list
	       bool found = false;
	       QueueItem *q = (QueueItem *)queue->firstChild();
	       while(q)
	       {
		    if(q->GDL() == gdl && q->User() == llusers[i]) found = true;
		    q = (QueueItem*)q->nextSibling();

	       }
	       if(!found)
		    new QueueItem(queue,gdl,llusers[i],fname,QString::number(size),100.0f*((float)recSize/(float)size));
	  }
     }
   
     emit transferInfo(gdl,fname,size,recSize,last10,users,status);
}
